import logging

from scrapy.http import Request
from scrapy.exceptions import IgnoreRequest
from scrapy.linkextractors import LinkExtractor

logger = logging.getLogger(__name__)


class LinkFilterMiddleware:
    """
    Downloader Middleware, or Spider Middleware,
    that allows a Scrapy Spider to filter requests.
    """

    def __init__(self, crawler):
        self.crawler = crawler
        self.debug = self.crawler.settings.getbool('LINK_FILTER_MIDDLEWARE_DEBUG', False)

    @classmethod
    def from_crawler(cls, crawler):
        return cls(crawler)

    def process_request(self, request, spider):
        """ Called as a Downloader Middleware """
        extractor = self._create_extractor(spider, request)
        if not extractor:
            return

        if not extractor.matches(request.url):
            if self.debug:
                logger.debug('Dropping request: %s', request, extra={'spider': spider})
            self.crawler.stats.inc_value('link_filtering/d/dropped_requests')
            raise IgnoreRequest("Link doesn't match extract rules")

    def process_spider_output(self, response, result, spider):
        """ Called as a Spider Middleware """
        extractor = self._create_extractor(spider, response)

        def _filter(request):
            if extractor and isinstance(request, Request) and not extractor.matches(request.url):
                if self.debug:
                    logger.debug('Dropping request: %s', request, extra={'spider': spider})
                self.crawler.stats.inc_value('link_filtering/s/dropped_requests')
                return False
            return True

        return (r for r in result or () if _filter(r))

    def _create_extractor(self, spider, request):
        """
        Use the extraction rules defined per URL and return a new LinkExtractor.
        """
        rules = {}
        # Use the spider rules first
        if isinstance(getattr(spider, 'extract_rules', False), dict):
            rules = spider.extract_rules
        # Update the spider rules with rules from the request
        if isinstance(request.meta.get('extract_rules'), dict):
            rules.update(request.meta['extract_rules'])
        if not rules:
            return
        return LinkExtractor(**rules)


DOWNLOADER_MIDDLEWARES = {
    # It goes before RobotsTxtMiddleware: 100
    'scrapy_link_filter.middleware.LinkFilterMiddleware': 50
}

SPIDER_MIDDLEWARES = {
    # It goes after DepthMiddleware: 900
    'scrapy_link_filter.middleware.LinkFilterMiddleware': 950
}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages

# https://github.com/kennethreitz/setup.py ✨ 🍰 ✨

NAME = 'scrapy-link-filter'
DESCRIPTION = 'Spider Middleware that allows a Scrapy Spider to filter requests.'
KEYWORDS = 'scrapy link filter'
URL = 'https://github.com/croqaz/scrapy-link-filter'
AUTHOR = 'Cristi Constantin'
EMAIL = 'cristi.constantin@live.com'

REQUIRES_PYTHON = '>=3.6.0'
REQUIRED = []

here = os.path.abspath(os.path.dirname(__file__))
about = {}

with open(os.path.join(here, NAME.replace('-', '_'), '__version__.py')) as f:
    exec(f.read(), about)

with open(os.path.join(here, 'README.md')) as f:
    long_description = f.read()

setup(
    version=about['__version__'],
    name=NAME,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=KEYWORDS,
    url=URL,
    author=AUTHOR,
    author_email=EMAIL,
    license='BSD',
    packages=find_packages(),
    platforms=['Any'],
    python_requires=REQUIRES_PYTHON,
    install_requires=REQUIRED,
    classifiers=[
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development',
        'Framework :: Scrapy',
    ],
)
